#!/usr/bin/env node
/*!
 * gitmars v4.0.0-alpha.8
 * 这是一个git工作流工具
 * (c) 2021-2022 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import dayjs from 'dayjs';
import inquirer from 'inquirer';
import sh from 'shelljs';
import chalk from 'chalk';
import { queue } from '@gitmars/core/lib/queue';
import getIsGitProject from '@gitmars/core/lib/git/getIsGitProject';
import getCurrentBranch from '@gitmars/core/lib/git/getCurrentBranch';
import { createArgs } from '@gitmars/core/lib/utils/command';
import echo from '@gitmars/core/lib/utils/echo';
import { getRevertCache, delRevertCache } from '@gitmars/core/lib/cache/revertCache';
import redoConfig from '#lib/conf/redo';
import i18n from '#lib/locales/index';

const { blue, green, red, yellow } = chalk;
const { args, options } = redoConfig;
if (!getIsGitProject()) {
  sh.echo(
    red(i18n.__("The current directory is not a git project directory"))
  );
  process.exit(1);
}
program.name("gitm redo").usage("[commitid...] [-m --mode [mode]]").description(i18n.__("Undo a commit record"));
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (commitid, opt) => {
  const current = getCurrentBranch();
  let revertCache = getRevertCache(current), cmd = [], commitIDs = [], mode = "";
  mode = " -m " + Math.abs(Number(opt.mode || 1));
  if (commitid.length > 0) {
    revertCache = revertCache.filter(
      (item) => commitid.some((id) => item.after["%H"].includes(id))
    );
  }
  if (revertCache.length === 0) {
    echo(yellow("\u6CA1\u6709\u627E\u5230\u53EF\u6062\u590D\u7684\u64A4\u9500\u8BB0\u5F55\uFF0C\u8FDB\u7A0B\u5DF2\u9000\u51FA"));
    process.exit(0);
  }
  const prompt = {
    type: "checkbox",
    message: i18n.__("Please select the undo record to restore"),
    name: "commitIDs",
    choices: []
  };
  revertCache.forEach(({ after }, index) => {
    const _time = dayjs(after["%aI"]).format("YYYY/MM/DD HH:mm");
    prompt.choices.push({
      name: `${green(index + 1 + ".")} ${green(after["%s"])} | ${yellow(
        after["%an"]
      )} | ${blue(_time)}`,
      value: after["%H"],
      checked: false
    });
  });
  commitIDs = (await inquirer.prompt(prompt)).commitIDs;
  if (commitIDs.length === 0) {
    echo(yellow("\u6CA1\u6709\u9009\u62E9\u4EFB\u4F55\u8BB0\u5F55\uFF0C\u8FDB\u7A0B\u5DF2\u9000\u51FA"));
    process.exit(0);
  }
  revertCache = revertCache.filter(
    (item) => commitIDs.includes(item.after["%H"])
  );
  cmd = revertCache.map((item) => ({
    cmd: `git revert -s --no-edit ${item.after["%H"]}${mode}`,
    config: {
      again: false,
      success: `\u64A4\u9500\u6210\u529F\uFF1A${item.after["%s"]}`,
      fail: i18n.__(
        "An error has occurred, please follow the instructions"
      )
    }
  }));
  queue(cmd).then(() => {
    delRevertCache(commitIDs);
  });
});
program.parse(process.argv);
